state = require("/include/stateswitcher")
tween = require("/include/tween")
cron = require("/include/cron")

--window dimensions
local WWIDTH = 960
local WHEIGHT = 640
local CIRCLE_RAD = 150
local CIRCLE_RAD_SQUARED = 22500

local controlMode = 1 -- 1 : Mouse - 2 : Keyboard

--initial graphics setup
love.graphics.setBackgroundColor(104, 136, 248) --set the background color to a nice blue
love.window.setMode(WWIDTH, WHEIGHT)
love.mouse.setVisible(false)

speed = 400
love.physics.setMeter(64) --the height of a meter our worlds will be 64px
world = love.physics.newWorld(0, 9.81*64, true) --create a world for the bodies to exist in with horizontal gravity of 0 and vertical gravity of 9.81

objects = {} -- table to hold all our physical objects

objects.zone = {}
objects.zone.x = 480
objects.zone.y = 320
objects.zone.radius = CIRCLE_RAD

--let's create a ball
objects.ball = {x = 480, y = 320, radius = 20, speed = 4}

objects.enemy = {}

spawnTimer = cron.every(0.2, function()
  local tempEnemy = {}
  --Pos defining code
  tempEnemy.speed = math.random (200, 400)
  tempEnemy.side = math.random(1, 4) -- 1 : Left - 2 : Up - 3 : Right - 4 : Down
  
  if tempEnemy.side == 1 or tempEnemy.side == 3 then --left or right
    tempEnemy.y = math.random(0, WHEIGHT)
    if tempEnemy.side == 1 then tempEnemy.x = -10; elseif tempEnemy.side == 3 then tempEnemy.x = WWIDTH+10; end
  else
    tempEnemy.x = math.random(0, WWIDTH)
    if tempEnemy.side == 2 then tempEnemy.y = -10; elseif tempEnemy.side == 4 then tempEnemy.y = WHEIGHT+10; end
  end
  --E/OF
  tempEnemy.radius = 10
  tempEnemy.isMoving = true
  tempEnemy.xDir = objects.ball.x-tempEnemy.x
  tempEnemy.yDir = objects.ball.y-tempEnemy.y
  if tempEnemy.xDir > 0 then tempEnemy.xSign = 1 else tempEnemy.xSign = -1 end
  if tempEnemy.yDir > 0 then tempEnemy.ySign = 1 else tempEnemy.ySign = -1 end
  if math.abs(tempEnemy.xDir) > math.abs(tempEnemy.yDir) then 
    tempEnemy.yDir = tempEnemy.yDir/tempEnemy.xDir; tempEnemy.xDir = 1;
    tempEnemy.xDir = tempEnemy.xDir*tempEnemy.xSign; tempEnemy.yDir = tempEnemy.yDir*tempEnemy.xSign;
  elseif math.abs(tempEnemy.yDir) > math.abs(tempEnemy.xDir) then
    tempEnemy.xDir = tempEnemy.xDir/tempEnemy.yDir; tempEnemy.yDir = 1;
    tempEnemy.xDir = tempEnemy.xDir*tempEnemy.ySign; tempEnemy.yDir = tempEnemy.yDir*tempEnemy.ySign;
  else
    tempEnemy.xDir = 1; tempEnemy.yDir = 1;
    tempEnemy.xDir = tempEnemy.xDir*tempEnemy.xSign; tempEnemy.yDir = tempEnemy.yDir*tempEnemy.ySign;
  end
  objects.enemy[#objects.enemy+1] = tempEnemy
end)

function love.update(dt)
  spawnTimer:update(dt)

  if controlMode == 1 then
    local mouseX, mouseY = love.mouse.getPosition()
    objects.ball.x, objects.ball.y = mouseX, mouseY
    if math.pow(mouseX-WWIDTH/2, 2) + math.pow(mouseY-WHEIGHT/2, 2) > CIRCLE_RAD_SQUARED then
      local angle = math.atan2(objects.ball.y-WHEIGHT/2, objects.ball.x-WWIDTH/2 ) -- angle of mouse in radians
      objects.ball.x, objects.ball.y = math.cos( angle ) * CIRCLE_RAD+WWIDTH/2, math.sin( angle ) * CIRCLE_RAD+WHEIGHT/2 -- clamp mouse value to circle radius
    end
  elseif controlMode == 2 then
    --here we are going to create some keyboard events
    if love.keyboard.isDown("left") then
      objects.ball.xVel = -1
    elseif love.keyboard.isDown("right") then
      objects.ball.xVel = 1
    else
      objects.ball.xVel = 0
    end
    
    if love.keyboard.isDown("up") then
      objects.ball.yVel = -1
    elseif love.keyboard.isDown("down") then
      objects.ball.yVel = 1
    else
      objects.ball.yVel = 0
    end
    
    local prevX, prevY = objects.ball.x, objects.ball.y
    objects.ball.x, objects.ball.y = objects.ball.x + objects.ball.xVel*objects.ball.speed, objects.ball.y + objects.ball.yVel*objects.ball.speed
    if math.sqrt(math.pow(480-objects.ball.x, 2)+math.pow(320-objects.ball.y, 2)) > objects.zone.radius then
      objects.ball.x = prevX
      objects.ball.y = prevY
    end
  end

  for i = 1, #objects.enemy do
    if objects.enemy[i].isMoving then
      if (((objects.enemy[i].side == 1 or objects.enemy[i].side == 3) and (objects.enemy[i].x > WWIDTH+50 or objects.enemy[i].x < -50)) or ((objects.enemy[i].side == 2 or objects.enemy[i].side == 4) and (objects.enemy[i].y > WHEIGHT+50 or objects.enemy[i].y < -50))) then
        objects.enemy[i].isMoving = false
      else
        objects.enemy[i].x = objects.enemy[i].x + objects.enemy[i].xDir*objects.enemy[i].speed/100
        objects.enemy[i].y = objects.enemy[i].y + objects.enemy[i].yDir*objects.enemy[i].speed/100
      end
    end
  end
  
end

function love.draw()
  love.graphics.setColor(50, 50, 50)
  love.graphics.circle("fill", objects.zone.x, objects.zone.y, objects.zone.radius)
  
  love.graphics.setColor(200, 0, 0)
  for i = 1, #objects.enemy do
    if objects.enemy[i].isMoving then
      love.graphics.circle("fill", objects.enemy[i].x, objects.enemy[i].y, objects.enemy[i].radius)
    end
  end
  
  love.graphics.setColor(255, 255, 255) --set the drawing color to red for the ball
  love.graphics.circle("fill", objects.ball.x, objects.ball.y, objects.ball.radius)
  
  love.graphics.print(tostring(#objects.enemy).."   enemies", 100, 100)
end

function love.keyreleased(key)
  if key == "tab" then
    local state = not love.mouse.isGrabbed()   -- the opposite of whatever it currently is
    love.mouse.setGrabbed(state)
  elseif key == "l" then
    if controlMode == 1 then controlMode = 2 else controlMode = 1 end
  end
end